import { useEffect, useState } from 'react';
import useEvent from 'react-use-event-hook';

function refsInitialState() {
  return {
    proms: []
  };
}

function useStateWithPromise(initialState) {
  const [state, setState] = useState(initialState);
  const [$refs] = useState(refsInitialState);
  useEffect(() => {
    if (!$refs.proms.length) return;
    const proms = $refs.proms.splice(0, $refs.proms.length);
    proms.forEach(prom => prom.nextState === state ? prom.resolve(state) : prom.reject(state));
  }, [$refs, state]);
  const setStateWithPromise = useEvent(stateAction => {
    let prom = {};

    if (typeof stateAction === 'function') {
      setState(prevState => {
        prom.nextState = stateAction(prevState);
        return prom.nextState;
      });
    } else {
      prom.nextState = stateAction;
    }

    return new Promise((resolve, reject) => {
      prom.resolve = resolve;
      prom.reject = reject;
      $refs.proms.push(prom);
    });
  });
  return [state, setStateWithPromise];
}

export default useStateWithPromise;